@@ -19,6 +19,21 @@
  *****************************************************************************/
 
 #define _GNU_SOURCE /* for getsid() */
+
+#include <unistd.h>
+#include <string.h>
+#include <stdio.h>
+#include <sysexits.h>
+#include <signal.h>
+#include <npapi.h>
+#include <fcntl.h>
+#include <sys/wait.h>
+#include <errno.h>
+#include <X11/X.h>
+#include <X11/Xutil.h>
+#include <X11/Xatom.h>
+#include <sys/socket.h>
+
 #include "mozplugger.h"
 
 static Display * display = 0;
@@ -56,6 +71,19 @@ static Atom windowOwnerMark;
 
 #define WINDOW (parentDetails.window)
 
+/*****************************************************************************/
+/**
+ * @brief Error handler callback
+ *
+ * Callback from X when there is an error. If debug defined error message is
+ * printed to debug log, otherise nothing is done
+ *
+ * @param[in] dpy The display pointer
+ * @param[in] err Pointer to the error event data
+ *
+ * @return Always returns zero
+ *
+ *****************************************************************************/
 static int error_handler(Display *dpy, XErrorEvent *err)
 {
 #ifdef DEBUG
@@ -67,8 +95,12 @@ static int error_handler(Display *dpy, XErrorEvent *err)
      return 0;
 }
 
-/*****************************************************************************
+/*****************************************************************************/
+/**
+ * @brief Initialise semaphore
+ *
  * Swallow mutex semaphore protection
+ *
  *****************************************************************************/
 static void initSwallowMutex(void)
 {
@@ -85,6 +117,15 @@ static void initSwallowMutex(void)
     windowOwnerMark = XInternAtom(display, "MOZPLUGGER_OWNER", 0);
 }
 
+/*****************************************************************************/
+/**
+ * @brief Get Host ID
+ *
+ * Construct a host ID using host name as source of information
+ *
+ * @return The host ID
+ *
+ *****************************************************************************/
 static uint32 getHostId(void)
 {
     char hostName[128];
@@ -108,8 +149,23 @@ static uint32 getHostId(void)
     return id;
 }
 
-static int getSwallowMutexOwner(uint32 * hostId, uint32 * pid)
-{   
+/*****************************************************************************/
+/**
+ * @brief Extract host Id and pid from window property
+ *
+ * This is common code used in various places and is factored out due to 
+ * complexity of 64 bit versus 32 bit machines.
+ *
+ * @param[in]  w      The window to get property from
+ * @param[in]  name   The name of property to get 
+ * @param[out] hostId The ID of the host currently owning the mutex
+ * @param[out] pid    The process ID of the mutex
+ *
+ * @return 1(true) if owner found, 0 otherwise
+ *
+ *****************************************************************************/
+static int getOwnerFromProperty(Window w, Atom name, uint32 * hostId, uint32 * pid)
+{
     unsigned long nitems;
     unsigned long bytes;      
     int fmt;
@@ -121,7 +177,7 @@ static int getSwallowMutexOwner(uint32 * hostId, uint32 * pid)
        this display! - watch out for the bizarre 64-bit platform behaviour
        where property returned is actually an array of 2 x  64bit ints with
        the top 32bits set to zero! */
-    XGetWindowProperty(display, wattr.root, swallowMutex,
+    XGetWindowProperty(display, w, name,
                        0, 2, 0, XA_INTEGER,
                        &type, &fmt, &nitems, &bytes,
                        &property);
@@ -146,16 +202,51 @@ static int getSwallowMutexOwner(uint32 * hostId, uint32 * pid)
     return success;
 }
 
+/*****************************************************************************/
+/**
+ * @brief Get owner of mutex semaphore
+ *
+ * Returns the current owner of the semaphore
+ *
+ * @param[out] hostId The ID of the host currently owning the mutex
+ * @param[out] pid    The process ID of the mutex
+ *
+ * @return 1(true) if owner found, 0 otherwise
+ *
+ *****************************************************************************/
+static int getSwallowMutexOwner(uint32 * hostId, uint32 * pid)
+{   
+    return getOwnerFromProperty(wattr.root, swallowMutex, hostId, pid);
+}
+
+/*****************************************************************************/
+/**
+ * @brief Set owner of mutex semaphore
+ *
+ * Sets the current owner of the semaphore
+ *
+ * @param[in] hostId The ID of the new owner of the mutex
+ * @param[in] pid    The process ID of the mutex
+ *
+ *****************************************************************************/
 static void setSwallowMutexOwner(uint32 hostId, uint32 pid)
 {
     unsigned long temp[2] = {hostId, pid};
-    D("Setting swallow mutex owner, hostId = 0x%08X, pid=%d\n", hostId, pid); 
+    D("Setting swallow mutex owner, hostId = 0x%08X, pid=%u\n", 
+                                            (unsigned) hostId, (unsigned) pid); 
 
     XChangeProperty(display, wattr.root, swallowMutex,
                         XA_INTEGER, 32, PropModeAppend,
                         (unsigned char*) (&temp), 2);
 }
 
+/*****************************************************************************/
+/**
+ * @brief Take mutex semaphore
+ *
+ * Takes ownership of the mutex semaphore
+ *
+ *****************************************************************************/
 static void takeSwallowMutex(void)
 {
     int countDown;
@@ -231,7 +322,13 @@ static void takeSwallowMutex(void)
     }
 }
 
-
+/*****************************************************************************/
+/**
+ * @brief Give the mutex semaphore
+ *
+ * Gives away ownership of the mutex semaphore
+ *
+ *****************************************************************************/
 static void giveSwallowMutex()
 {
     if((display == 0) || (swallowMutex == 0) || (swallowMutexTaken ==0))
@@ -242,40 +339,33 @@ static void giveSwallowMutex()
     swallowMutexTaken = 0;
 }
 
-/*****************************************************************************
+/*****************************************************************************/
+/**
+ * @brief Check and Mark
+ *
  * Mark the victim window with a property that indicates that this instance
  * of mozplugger-helper has made a claim to this victim. This is used to 
  * guard against two instances of mozplugger-helper claiming the same victim.
+ *
+ * @param[in] w The window
+ *
+ * @return True(1) if taken, False(0) if not
+ *
  *****************************************************************************/
 static int chkAndMarkVictimWindow(Window w)
 {
-     unsigned long nitems;
-     unsigned long bytes;      
-     int fmt;
-     Atom type;
-     unsigned char * property = NULL;
      unsigned long temp[2];
      uint32 ourPid;
      uint32 ourHostId;
+
+     uint32 pid;
+     uint32 hostId;
      int gotIt = 0;
 
      /* See if some other instance of Mozplugger has already 'marked' this
       * window */
-     XGetWindowProperty(display, w, windowOwnerMark,
-                        0, 2, 0, XA_INTEGER,
-                        &type, &fmt, &nitems, &bytes,
-                        &property);
-
-     if(property)
+     if(getOwnerFromProperty(w, windowOwnerMark, &hostId, &pid))
      {
-          /* Just check all is correct! */
-	  if((type != XA_INTEGER) || (fmt!=32) || (nitems!=2))
-          {
-	       D("XGetWindowProperty(windowOwnerMark) returned bad values " 
-                                   "%ld,%d,%lu,%lu\n", (long) type, fmt, nitems, 
-                                   bytes);
-          }
-  	  XFree(property);
           return 0;          /* Looks like someone else has marked this window */
      }
 
@@ -291,27 +381,12 @@ static int chkAndMarkVictimWindow(Window w)
                      (unsigned char*) (&temp), 2);
 
      /* See if we got it */
-     XGetWindowProperty(display, w, windowOwnerMark,
-                        0, 2, 0, XA_INTEGER,
-                        &type, &fmt, &nitems, &bytes,
-                        &property);
-
-     if(property)
+     if(getOwnerFromProperty(w, windowOwnerMark, &hostId, &pid))
      {
-
-          /* Just check all is correct! */
-          if((type != XA_INTEGER) || (fmt!=32) || (nitems!=2))
-          {
-               fprintf(stderr, "XGetWindowProperty returned bad values " 
-                                   "%ld,%d,%lu,%lu\n", (long) type, fmt, nitems, 
-                                   bytes);
-          }
-          else if( ( ((unsigned long *)property)[1] == ourPid)
-                  && ( ((unsigned long *)property)[0] == ourHostId) )
+          if( (pid == ourPid) && (hostId == ourHostId) )
           {
                gotIt = 1;
           }
-  	  XFree(property);
      }
      else         
      {
@@ -329,6 +404,18 @@ static int chkAndMarkVictimWindow(Window w)
 static int xaspect;
 static int yaspect;
 
+/*****************************************************************************/
+/**
+ * @brief Calculate scaling factor
+ *
+ * Give a and b calculate a number that both a and b can be divided by
+ *
+ * @param[in] a  First value 
+ * @param[in] b  Second value
+ *
+ * @return Scaling factor
+ *
+ *****************************************************************************/
 static int gcd(int a, int b)
 {
      if (a < b) return gcd(b,a);
@@ -336,6 +423,18 @@ static int gcd(int a, int b)
      return gcd(b, a % b);
 }
 
+/*****************************************************************************/
+/**
+ * @brief Set aspect ratio
+ *
+ * Store away the aspect ratio
+ *
+ * @param[in] x  X value 
+ * @param[in] y  Y value
+ *
+ * @return 1(true) if change has occurred
+ *
+ *****************************************************************************/
 static int set_aspect(int x, int y)
 {
      int ox = xaspect;
@@ -347,6 +446,13 @@ static int set_aspect(int x, int y)
      return (ox != xaspect || oy != yaspect);
 }
 
+/*****************************************************************************/
+/**
+ * @brief Adjust window size
+ *
+ * Changed size of window
+ *
+ *****************************************************************************/
 static void adjust_window_size(void)
 {
      int x = 0;
@@ -366,13 +472,14 @@ static void adjust_window_size(void)
 
      if (flags & H_FILL)
      {
-	  D("Resizing window %x with FILL\n", victimDetails.window);
+	  D("Resizing window %x with FILL\n", (unsigned) victimDetails.window);
      }
      else if (flags & H_MAXASPECT)
      {
 	  if (xaspect && yaspect)
 	  {
-	       D("Resizing window %x with MAXASPECT\n", victimDetails.window);
+	       D("Resizing window %x with MAXASPECT\n", 
+                                               (unsigned) victimDetails.window);
 	       tmph = h / yaspect;
 	       tmpw = w / xaspect;
 	       if (tmpw < tmph) tmph = tmpw;
@@ -423,21 +530,24 @@ static void adjust_window_size(void)
           XSendEvent(display, victimDetails.window, False, StructureNotifyMask,
                   &event);
      }
-     else
-     {
-          XMoveResizeWindow(display, victimDetails.window, 
-                                     x, y, (unsigned)w, (unsigned)h);
-          victimDetails.x = x;
-          victimDetails.y = y;
-          victimDetails.width = w;
-          victimDetails.height = h;
-    }
+
+     /* Always resize the window, even when the target size has not changed.
+	This is needed for gv. */
+     XMoveResizeWindow(display, victimDetails.window, 
+		       x, y, (unsigned)w, (unsigned)h);
+     victimDetails.x = x;
+     victimDetails.y = y;
+     victimDetails.width = w;
+     victimDetails.height = h;
 }
 
-/*****************************************************************************
- * Window reparenting
+/*****************************************************************************/
+/**
+ * @brief Change window leader
+ *
+ * Change the leader of the window
+ *
  *****************************************************************************/
-
 static void change_leader(void)
 {
      XWMHints *leader_change;
@@ -450,6 +560,15 @@ static void change_leader(void)
      }
 }
 
+/*****************************************************************************/
+/**
+ * @brief Reparent window
+ *
+ * Reparent the window
+ *
+ * @return 1(true) if success, 0(false) if not
+ *
+ *****************************************************************************/
 static int reparent_window(void)
 {
      if (!victimDetails.window) return 0;
@@ -458,10 +577,11 @@ static int reparent_window(void)
      XSelectInput(display, victimDetails.window, StructureNotifyMask); 
      XSync(display, False);
 
-     D("Changing leader of window %x\n", victimDetails.window);
+     D("Changing leader of window %x\n", (unsigned) victimDetails.window);
      change_leader();
      
-     D("Reparenting window %x into %x\n", victimDetails.window, WINDOW);
+     D("Reparenting window %x into %x\n", (unsigned)victimDetails.window, 
+                                                             (unsigned)WINDOW);
      XReparentWindow(display, victimDetails.window, WINDOW, 0, 0);
 
      D("reparent_window() done\n");
@@ -469,6 +589,18 @@ static int reparent_window(void)
      return 1;
 }
 
+/*****************************************************************************/
+/**
+ * @brief Is window viewable
+ *
+ * Check if window is viewable by the user
+ *
+ * @param[in] display Pointer to the display
+ * @param[in] w       The window
+ *
+ * @return 1(true) if viewable, 0(false) if not
+ *
+ *****************************************************************************/
 static int is_viewable(Display *display, Window w)
 {
      XWindowAttributes a;
@@ -485,9 +617,19 @@ static int is_viewable(Display *display, Window w)
      }
      return 1;
 }
-/******************************************************************************
+
+/******************************************************************************/
+/**
+ * @brief Wrapper for strcmp
+ *
  * Traditionally strcmp returns -1, 0 and +1 depending if name comes before
  * or after windowname, this function also returns +1 if error
+ *
+ * @param[in] windowname The window name to compare against
+ * @param[in] name       The name to compare against window name
+ *
+ * @return 0 if match, -1/+1 if different 
+ *
  *****************************************************************************/
 static int my_strcmp(char *windowname, char *name)
 {
@@ -509,6 +651,18 @@ static int my_strcmp(char *windowname, char *name)
      }
 }
 
+/******************************************************************************/
+/**
+ * @brief Check window name
+ *
+ * Check name against the name of the passed window
+ *
+ * @param[in] w      The window to compare against
+ * @param[in] name   The name to compare against window name
+ *
+ * @return 1 if match, 0 if not 
+ *
+ *****************************************************************************/
 static char check_window_name(Window w, char *name)
 {
      char *windowname;
@@ -519,7 +673,7 @@ static char check_window_name(Window w, char *name)
      if (XFetchName(display, w, &windowname))
      {
 	  D("Winrecur, checking window NAME %x (%s != %s)\n",
-	    w, windowname, name);
+	    (unsigned)w, windowname, name);
 	  match = (my_strcmp(windowname, name) == 0);
 	  XFree(windowname);
 	  if (match) {
@@ -531,7 +685,7 @@ static char check_window_name(Window w, char *name)
      if (XGetClassHint(display, w, &windowclass))
      {
 	  D("Winrecur, checking window CLASS %x (%s != %s)\n",
-	    w, windowclass.res_name, name);
+	    (unsigned)w, windowclass.res_name, name);
 	  match = (my_strcmp(windowclass.res_name, name) == 0);
 	  XFree(windowclass.res_class);
 	  XFree(windowclass.res_name);
@@ -545,11 +699,16 @@ static unsigned int ignore_window_count = 0;
 static Window ignored[MAX_IGNORED];
 
 /*****************************************************************************
- * init_winrecur
+ * @brief initialise recurwivw search for windows
  *
  * This function should be called before forking the application. It runs 
  * through all windows on the screen to a depth of 'depth' and records their
  * identify in the ignored list. 
+ *
+ * @param[in] from  The current window root
+ * @param[in] depth The number of windows down to go
+ * @param[in] name  The name of the window being looked for
+ *
  *****************************************************************************/
 static void init_winrecur(Window from, int depth, char * name)
 {
@@ -559,19 +718,15 @@ static void init_winrecur(Window from, int depth, char * name)
      unsigned int num_children;
 
      D("init_winrecur: storing family tree of window %x (depth=%d)\n", 
-                                                          from, depth);
+                                                       (unsigned)from, depth);
      
-     if (check_window_name(from, name))
+     if (ignore_window_count < MAX_IGNORED) 
      {
-          if (ignore_window_count < MAX_IGNORED) 
-          {
-               ignored[ignore_window_count++] = from;
-          }
-          else
-          {
-               D("Woo too many windows!\n");
-          }
-          return;
+          ignored[ignore_window_count++] = from;
+     }
+     else
+     {
+          D("Woo too many windows!\n");
      }
 
      if (depth > 0)
@@ -592,11 +747,19 @@ static void init_winrecur(Window from, int depth, char * name)
      }
 }
 
-/*****************************************************************************
- * find_winrecur
+/*****************************************************************************/
+/**
+ * @brief Perform recursive find of windows
  *
  * This function should be called after forking the application and tries to
  * see if this window (or its child) are the ones to swallow. 
+ *
+ * @param[in] from  The current window root
+ * @param[in] depth The number of windows down to go
+ * @param[in] name  The name of the window being looked for
+ *
+ * @return 0 no window found, otherwise the window ID
+ *
  *****************************************************************************/
 static Window find_winrecur(Window from, int depth, char *name)
 {
@@ -605,17 +768,8 @@ static Window find_winrecur(Window from, int depth, char *name)
      unsigned int e;
      unsigned int num_children;
 
-     D("find_winrecur, checking window %x (depth=%d)\n",from,depth);
+     D("find_winrecur, checking window %x (depth=%d)\n",(unsigned)from,depth);
      
-     for (e=0; e < ignore_window_count; e++) 
-     {
-	  if (ignored[e] == from)
-          {
-               /* This window is OLD, ignore it (and its children) */
-	       return (Window)0;
-          }
-     }
-
      /* This window doesn't match our 'HIDDEN' criteria, ignore it
       * (and its children) */
      if (!(flags & H_HIDDEN) && !is_viewable(display, from))
@@ -623,6 +777,15 @@ static Window find_winrecur(Window from, int depth, char *name)
   
      if (check_window_name(from, name))
      {
+          for (e=0; e < ignore_window_count; e++) 
+          {
+	       if (ignored[e] == from)
+               {
+                    /* This window is OLD, ignore it (and its children) */
+	            return (Window)0;
+               }
+          }
+
           /* See if some instance of mozplugger got the window before us, if
            * not mark it as ours */
           if(chkAndMarkVictimWindow(from))
@@ -659,42 +822,80 @@ static Window find_winrecur(Window from, int depth, char *name)
      return (Window)0;
 }
 
+/*****************************************************************************/
+/**
+ * @brief Save the initial window state
+ * 
+ * Create a list of current windows.
+ *
+ *****************************************************************************/
 static void save_initial_windows(void)
 {
      ignore_window_count = 0;
      init_winrecur(wattr.root, 3, winname);
 }
 
+/*****************************************************************************/
+/**
+ * @brief Setup display
+ *
+ * setup display ready for any reparenting of the application window. If the
+ * WINDOW is NULL (can happen see mozdev bug #18837), then return failed
+ * code from this function to indicate no window available. This then means
+ * no swallowing will occur (even if requested).
+ *
+ * @return 1(true) if success, 0(false) if not
+ *
+ *****************************************************************************/
 static int setup_display(void)
 {
-     char *displayname = getenv("DISPLAY");
+     char *displayname;
+
+     if(WINDOW == 0)       /* mozdev bug #18837 */
+     {
+          D("setup_display() WINDOW is Null - so nothing setup\n");
+          return 1;
+     }
+
+     displayname = getenv("DISPLAY");
      D("setup_display(%s)\n", displayname);
 
      XSetErrorHandler(error_handler);
 
      if (!(display = XOpenDisplay(displayname)))
+     {
+          D("setup_display() failed cannot open display!!\n");
 	  return 1;
+     }
 
      if (!XGetWindowAttributes(display, WINDOW, &wattr))
+     {
+          D("setup_display() failed cannot get window attributes!!\n");
+          XCloseDisplay(display);
+          display = 0;
           return 1;
+     }
 
-     D("display=%x\n", display);
-     D("WINDOW=%x\n", WINDOW);
-     D("rootwin=%x\n", wattr.root);
+     D("display=%x\n", (unsigned) display);
+     D("WINDOW=%x\n", (unsigned) WINDOW);
+     D("rootwin=%x\n", (unsigned) wattr.root);
      
      D("setup_display() done\n");
 
      return 0;
 }
 
-/*****************************************************************************
- * Wrapper for execlp() that calls the application.
+/*****************************************************************************/
+/** @brief Wrapper for execlp() that calls the application.
  *
  * WARNING: This function runs in the daughter process so one must assume the
  * daughter uses a copy (including) heap memory of the parent's memory space
  * i.e. any write to memory here does not affect the parent memory space. 
  * Since Linux uses copy-on-write, best leave memory read-only and once execlp
  * is called, all daughter memory is wiped anyway (except the stack).
+ *
+ * @param[in] argv Pointer to list of arguments
+ *
  *****************************************************************************/
 static void run_app(char **argv)
 {
@@ -747,10 +948,14 @@ static void run_app(char **argv)
      exit(EX_UNAVAILABLE);
 }
 
-
-/*****************************************************************************
+/*****************************************************************************/
+/**
+ * @brief Find victim
+ *
  * Looks for the window and reparents it when found.
  *
+ * @param[in] window The window
+ *
  *****************************************************************************/
 static void find_victim(Window window)
 {
@@ -765,7 +970,7 @@ static void find_victim(Window window)
 	       XWindowAttributes ca;
 	       XGetWindowAttributes(display, victimDetails.window, &ca);
 
-   	       D("Found it!! Victim=%x\n", victimDetails.window);
+   	       D("Found it!! Victim=%x\n", (unsigned) victimDetails.window);
                victimDetails.borderWidth = ca.border_width;
                victimDetails.x = ca.x;
                victimDetails.y = ca.y;
@@ -785,11 +990,31 @@ static void find_victim(Window window)
      }
 }
 
+/******************************************************************************/
+/**
+ * @brief Callback for filtering X events
+ *
+ * Called from X to filter events, in this case always returns true
+ *
+ * @param[in] dpy Pointer to display
+ * @param[in] ev  Pointer to event
+ * @param[in] arg Pointer to any event data
+ *
+ * @return Always returns true
+ *
+ *****************************************************************************/
 static Bool AllXEventsPredicate(Display *dpy, XEvent *ev, char *arg)
 {
      return True;
 }
 
+/******************************************************************************/
+/**
+ * @brief Check events from X
+ *
+ * Read in events from X and process
+ *
+ *****************************************************************************/
 static void check_x_events(void)
 {
      XEvent ev;
@@ -806,12 +1031,12 @@ static void check_x_events(void)
 		    break;
 
 	       case MapNotify:
-		    D("***MapNotify for root: %x\n", ev.xmap.window);
+		    D("***MapNotify for root: %x\n", (unsigned) ev.xmap.window);
 		    find_victim(ev.xmap.window);
 		    break;
 
 	       case ConfigureNotify:
-		    D("***ConfigureNotify for root: %x\n", ev.xconfigure.window);
+		    D("***ConfigureNotify for root: %x\n", (unsigned) ev.xconfigure.window);
 		    find_victim(ev.xconfigure.window);
 		    break;
 
@@ -876,7 +1101,7 @@ static void check_x_events(void)
 	       {
 	       case ConfigureRequest:
                     mask = ev.xconfigurerequest.value_mask;
-		    D("ConfigureRequest to WINDOW mask=0x%x\n", mask);
+		    D("ConfigureRequest to WINDOW mask=0x%lx\n", mask);
                     if(ev.xconfigurerequest.window != victimDetails.window)
                     {
                          D("- Strange Configure Request not for victim\n");
@@ -922,13 +1147,20 @@ static void check_x_events(void)
      }
 }
 
+/******************************************************************************/
+/**
+ * @brief Check events from pipe connected to mozplugger
+ *
+ * Read in events from pipe connected to mozplugger and process
+ *
+ *****************************************************************************/
 static void check_pipe_fd_events(void)
 {
      static NPWindow wintmp;
      int n;
   
      Window oldwindow = WINDOW;
-     D("Got pipe_fd data, old parent=%x pipe_fd=%d\n", WINDOW, pipe_fd);
+     D("Got pipe_fd data, old parent=%x pipe_fd=%d\n", (unsigned) WINDOW, pipe_fd);
   
      n = read(pipe_fd, ((char *)& wintmp),  sizeof(wintmp));
      if (n < 0)
@@ -953,7 +1185,7 @@ static void check_pipe_fd_events(void)
      parentDetails.width = (int) wintmp.width;
      parentDetails.height = (int) wintmp.height;
 
-     D("Got pipe_fd data, new parent=%x\n", WINDOW);
+     D("Got pipe_fd data, new parent=%x\n", (unsigned) WINDOW);
   
      if (WINDOW && WINDOW != oldwindow && display)
      {
@@ -968,6 +1200,13 @@ static void check_pipe_fd_events(void)
      adjust_window_size();
 }
 
+/******************************************************************************/
+/**
+ * @brief Check all events
+ *
+ * Use select to check if any events need processing
+ *
+ *****************************************************************************/
 static void check_all_events(void)
 {
      struct timeval tv;
@@ -1000,6 +1239,15 @@ static void check_all_events(void)
      }
 }
 
+/******************************************************************************/
+/**
+ * @brief handle the application
+ *
+ * Wait for application to finish and exit helper if a problem
+ *
+ * @param[in] pid process ID of the application
+ *
+ *****************************************************************************/
 static void handle_app(pid_t pid)
 {
      int status;
@@ -1041,7 +1289,11 @@ static void handle_app(pid_t pid)
      D("Exited OK!\n");
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief Exit early due to problem
+ *
+ * Prints a message to stderr and then exits the application
  *
  *****************************************************************************/
 static void exitEarly(void)
@@ -1052,7 +1304,14 @@ static void exitEarly(void)
 }
 
 /******************************************************************************
- * main() normally called from the child process started by mozplugger.so 
+ * @brief main() 
+ *
+ * normally called from the child process started by mozplugger.so 
+ *
+ * @param[in] argc The number of arguments
+ * @param[in] argv List of arguments
+ *
+ * @return Never returns (unless app exits)
  * 
  *****************************************************************************/
 int main(int argc, char **argv)
@@ -1074,8 +1333,8 @@ int main(int argc, char **argv)
 	    &repeats,
 	    &pipe_fd,
 	    &temp,
-	    &x,
-	    &y,
+	    &x,       /* Not needed and not used */
+	    &y,       /* Not needed and not used */
 	    &parentDetails.width,
 	    &parentDetails.height);
 
@@ -1120,8 +1379,11 @@ int main(int argc, char **argv)
 	  
 	  if (setup_display())
 	  {
-	       D("setup_display() failed\n");
-	       flags &=~ H_SWALLOW;
+               if(flags & H_SWALLOW)
+               { 
+	            D("Canceling swallow functionality\n");
+	            flags &=~ H_SWALLOW;
+               }
 	  }
 
 	  if (flags & H_SWALLOW)
@@ -1152,12 +1414,6 @@ int main(int argc, char **argv)
 	  if (pid == 0)
 	  {
                /* Child process */
-
-#if 0 /* Peter Leese - I'm sure this is not needed, especially if FD is closed 
-         in run_app() */
-               XSelectInput(display, WINDOW, 0);
-               XSelectInput(display, wattr.root, 0);
-#endif
 	       D("Running %s\n", command);
 	       run_app(app_argv);
 	  }
